#!/usr/bin/env ruby
$: << File.dirname( __FILE__ )
require 'rexml/document'

require 'zip/zip'
require 'zip/ziprequire'
require 'zip/stdrubyext'

require 'OOoException'

module OOo
  
  #--=======================================================================
  #--=======================================================================
  #++
  class Meta
    
    NAME_MAP = {  
      'description' => 'dc:description',
      'subject'     => 'dc:subject',
      'creator'     => 'dc:creator',
      'date'        => 'dc:date',
      'language'    => 'dc:language',
      'title'       => 'dc:title'     
     }

     VALUE_MAP = NAME_MAP.invert

    XPATH_BASE  = "*/office:meta"
     
     #--=======================================================================
     #++
     def keywords
       @doc.elements.to_a( "#{XPATH_BASE}/meta:keywords/meta:keyword" ).map{ |e|
         e.text
       }       
     end


     #--=======================================================================
     #++
     def find( xpath )
       begin
         return @doc.elements.to_a( xpath.to_s  )[0].text        
       rescue Exception
         raise OOo::OOoException.new( "Error finding with xpath '#{xpath}': #{$!}", $@ ) 
       end
     end
     
     
    #--=======================================================================
    #++
    def initialize( src )
      @doc = REXML::Document.new(  src.to_s )
    end
    
    #--=======================================================================
    #++
    def Meta.map_name n
       NAME_MAP[ n.to_s.strip ] ? NAME_MAP[ n.to_s.strip ] : n.to_s.strip
    end

    def lose_trailing_text( n )
       n.gsub( /[$!=]$/, '')  
    end
    
    def map_for_assignment n
      n = lose_trailing_text n.strip
      Meta.map_name n
      #      NAME_MAP.value?(  n.strip )  ?  VALUE_MAP[ n.strip ] : n.strip
    end

    #--=======================================================================
    #++
    def to_s
      @doc.to_s
    end
    

    #--=======================================================================
    #++
    def method_missing( name, *args )
      n = name.to_s
      if is_assignment? n  
        el = map_for_assignment n
        xpath = "#{XPATH_BASE}/#{el}"     
        assign( xpath, *args)  
      else
        el = Meta.map_name n
        xpath = "#{XPATH_BASE}/#{el}"     
        find( xpath  ) 
      end

    end

   private 
   #--=======================================================================
    #++
    def is_assignment?( name )
      name.strip =~ /=$/
    end
 
  
    #--=======================================================================
    #++
    def assign( xpath, val )
      node = @doc.elements.to_a( xpath )[0]
      node.text=val.to_s
    end    
  end
end
